from typing import Optional

from cpl_core.console import Console

from py_to_uxf_core.abc.class_scanner_abc import ClassScannerABC
from py_to_uxf_core.abc.file_scanner_abc import FileScannerABC
from py_to_uxf_core.abc.function_scanner_abc import FunctionScannerABC
from py_to_uxf_core.abc.python_parser_abc import PythonParserABC
from py_to_uxf_core.model.python_class import PythonClass


class PythonParserService(PythonParserABC):

    def __init__(
            self,
            file_scanner: FileScannerABC,
            class_scanner: ClassScannerABC,
            function_scanner: FunctionScannerABC
    ):
        PythonParserABC.__init__(self)

        self._file_scanner = file_scanner
        self._class_scanner = class_scanner
        self._function_scanner = function_scanner

    def parse(self):
        files = self._file_scanner.scan_files()
        for file in files:
            Console.write_line('\nfi:', file)
            is_comment = False
            with open(file, 'r') as file_content:
                cls: Optional[PythonClass] = None
                for line in file_content.readlines():
                    line = line.replace('    ', '')
                    line = line.replace('\t', '')
                    # replace line break at the end of line
                    if line.endswith('\n'):
                        line = line.replace('\n', '')

                    if line == '\n' or line == '':
                        continue

                    # one line comments
                    if line != '"""' and line.startswith('"""') and line.endswith('"""') or line.startswith('#'):
                        continue

                    # start multi line comment
                    if line.startswith('"""') and not is_comment:
                        is_comment = True
                        continue

                    # end multi line comment
                    if line.startswith('"""') or line.endswith('"""') and is_comment:
                        is_comment = False
                        continue

                    if is_comment:
                        continue

                    if cls is None:
                        cls = self._class_scanner.scan_line_for_classes(line)
                        if cls is not None:
                            Console.write_line('cl:', cls.name)
                        continue

                    func = self._function_scanner.scan_line_for_function(line)
                    if func is not None:
                        cls.add_function(func)
                        Console.write_line('fu:', func.access_modifier.value, func.name, func.args, func.return_type)
